{************************************************************}
{           CAD Importer SDK DLL Version demo          }
{                                                            }
{      Copyright (c) 2002-2015 SoftGold software company     }
{                                                            }
{************************************************************}
//{$DEFINE DRAWSPLINE}

unit Main;

interface

{$DEFINE SG_CADIMPORTERDLLDEMO}

//{$DEFINE MEMCHK}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ComCtrls, StdCtrls, Math, Menus, Buttons, sgConsts, xmldom,
  XMLIntf, msxmldom, XMLDoc,
  {$IFDEF MEMCHK}
  MemCheck,
  {$ENDIF}
  SGLines, sgLists, sgAboutDlg, sgInitSHX;

type
  ECADDLLException = class(Exception);

  TfmCADDLLdemo = class(TForm)
    pnlOptions: TPanel;
    OpenDialog1: TOpenDialog;
    MainMenu1: TMainMenu;
    mmiHome: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    mmiOpen: TMenuItem;
    mmiFitToWindow: TMenuItem;
    lblMeasure: TLabel;                 
    cbSplitArcs: TCheckBox;
    cbLayers: TComboBox;
    pmiAbout: TMenuItem;
    pmiShowAbout: TMenuItem;
    cbUseWinLine: TCheckBox;
    cbDimensions: TCheckBox;
    sbAllLayers: TCheckBox;
    cbProhibitArcsAsCurves: TCheckBox;
    btnFitToWindow: TSpeedButton;
    cbLayouts: TComboBox;
    cbTextAsCurves: TCheckBox;
    cbSplitSplines: TCheckBox;
    CheckBox1: TCheckBox;
    N2: TMenuItem;
    mmiFontSettings: TMenuItem;
    N3: TMenuItem;
    mmiSaveAs: TMenuItem;
    SaveDialog: TSaveDialog;
    mmiView: TMenuItem;
    mmiDoubleBuffered: TMenuItem;
    procedure FormPaint(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure cbSplitArcsClick(Sender: TObject);
    procedure cbLayersDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mmiOpenClick(Sender: TObject);
    procedure FitToWindowClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure pmiShowAboutClick(Sender: TObject);
    procedure cbProhibitArcsAsCurvesClick(Sender: TObject);
    procedure cbLayoutsChange(Sender: TObject);
    procedure cbUseWinLineClick(Sender: TObject);
    procedure cbTextAsCurvesClick(Sender: TObject);
    procedure cb3DFaceForACISClick(Sender: TObject);
    procedure cbSplitSplinesClick(Sender: TObject);
    procedure mmiFontSettingsClick(Sender: TObject);
    procedure mmiSaveAsClick(Sender: TObject);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure mmiDoubleBufferedClick(Sender: TObject);
  private
    FExtentsCAD: TF2DRect;
    FDrawRect: TF2DRect;
    FCurrentDir: string;
    FDownPos: TPoint;
    FDown: Boolean;
    FCADHandle: THandle;
    function GetVewport: TRect;
    function GetDrawRect: TF2DRect;
    function GetScale: Double;
    procedure DoFit;
    procedure UpdateExtents;
    function DoScale(const AScale: Double; ScreenPos: TPoint): Boolean;
    function GetAbsHeight: Double;
    function GetAbsWidth: Double;
  protected
    property DrawRect: TF2DRect read GetDrawRect;
    property Scale: Double read GetScale;
    property AbsWidth: Double read GetAbsWidth;
    property AbsHeight: Double read GetAbsHeight;
  public
    { Public declarations }
  end;

var
  fmCADDLLdemo: TfmCADDLLdemo;
  Lines: TsgLines;
  LScale: Double;

implementation

uses CADIntf, SHXDlg;

{$R *.DFM}

const
  cnstCaption = 'CAD Importer DLL [Delphi Demo]';

type
  TMyPaint = class(TPaintBox)
  end;

  PsgImageParam = ^TsgImageParam;
  TsgImageParam = packed record
    Heigth: Integer;
    Width: Integer;
    PixelFormat: TPixelFormat;
  end; 

var
  bSplittedSplines, bTextByCurves, b3DFaceForAcis: Boolean;
  EData: PcadData;
  IsInsideInsert: Boolean = False;

procedure Error;
var Buf: array[Byte] of Char;
begin
  CADGetLastError(Buf);
  raise ECADDLLException.Create(Buf);
end;

{ DrawPolyPolyLine

  Draws a polyline on points of the prepared list
  (list consists of PPoint-points).                 }
procedure DrawPolyPolyLine(DC: HDC; IntPoints: TList);
var
  Points: TsgIntegerList;
  I, N, C: Integer;
  P: Pointer;
  Pt1, Pt2, PtPrev: TPoint;
  isPrevDot: Boolean;
begin
  if DC = 0 then Exit;
  Points := TsgIntegerList.Create;
  try
    isPrevDot:= False;
    I := 0;
    while I < IntPoints.Count -1 do
    begin
      Pt1 :=  PPoint(IntPoints.Items[I])^;
      Pt2 :=  PPoint(IntPoints.Items[I+1])^;
      // For correct drawing and scaling dots
      if (Pt2.X = Pt1.X) and (Pt2.Y = Pt1.Y) then
      begin
        // If the current dot equally with previous dot
        if isPrevDot and (PtPrev.X = Pt1.X) and (PtPrev.Y = Pt1.Y) then
        begin
          Inc(I, 2);
          Continue;// miss of identical points
        end;
        PtPrev.X := Pt1.X;
        PtPrev.Y := Pt1.Y;
        Pt2.Y := Pt2.Y + 1;
        isPrevDot:= True;
      end
      else
        isPrevDot:= False;
      Points.Add(Pt1.X);
      Points.Add(Pt1.Y);
      Points.Add(Pt2.X);
      Points.Add(Pt2.Y);
      Inc(I, 2);
    end;
    N := Points.Count shr 2;
    if N = 0 then Exit;
    C := Points.Count;
    for I := 0 to N - 1 do
      Points.Add(2);
    P := Points.List;
    Inc(PInteger(P), C);
    PolyPolyline(DC, Points.List^, P^, N);
  finally
    Points.Free;
  end;
end;

{ DrawPolyLine

  Draws a only solid polyline on points of the prepared list
  (list consists of PPoint-points).                 }
procedure DrawPolyLine(DC: HDC; IntPoints: TList);
var
  Points: TsgIntegerList;
  I: Integer;
  Pt, PtPrev: TPoint;
begin
  if DC = 0 then Exit;
  Points := TsgIntegerList.Create;
  try
    for I := 0 to IntPoints.Count - 1 do
    begin
      Pt := PPoint(IntPoints.Items[I])^;
      if (I <> 0) and (PtPrev.X = Pt.X) and (PtPrev.Y = Pt.Y) then
        Continue;// miss of identical points
      Points.Add(Pt.X);
      Points.Add(Pt.Y);
      PtPrev.X := Pt.X;
      PtPrev.Y := Pt.Y;
    end;
    Windows.Polyline(DC, Points.List^, Points.Count shr 1);
  finally
    Points.Free;
  end;
end;

{ TDrawRect

  Converts FCADHandle units to pixels.}
function GetPoint(const P: TFPoint): TPoint;
var
  vDrawRect: TF2DRect;
begin
  vDrawRect := fmCADDLLdemo.DrawRect;
  Result.X := Round(vDrawRect.Left + (vDrawRect.Right - vDrawRect.Left) * (P.X - fmCADDLLdemo.FExtentsCAD.Left) / fmCADDLLdemo.AbsWidth);
  Result.Y := Round(vDrawRect.Top - (vDrawRect.Bottom - vDrawRect.Top) * (P.Y - fmCADDLLdemo.FExtentsCAD.Top) / fmCADDLLdemo.AbsHeight);
end;

{ GetPointsList

  Function converts the PFPoint-list in the PPoint-list
  with converting FCADHandle units to pixels. }
procedure GetPointsList(SingPoints: TFPointList; IntPts: TList);
var
  I: Integer;
  DXFPt: TFPoint;
  PIntPt: PPoint;
begin
  for I := 0 to SingPoints.Count - 1 do
  begin
    New(PIntPt);
    DXFPt := SingPoints[I];
    PIntPt.X := GetPoint(DXFPt).X;
    PIntPt.Y := GetPoint(DXFPt).Y;
    IntPts.Add(PIntPt);
  end;
end;

procedure DrawPoint(ACanvas: TCanvas);
var
  P: TPoint;
begin
  P := GetPoint(EData.Point);// EData.Point - point position
  ACanvas.Pixels[P.X, P.Y] := ACanvas.Pen.Color;
end;

{ DrawGlobal

  Draws a (poly)line in global coordinates. }
procedure DrawGlobal(ACanvas: TCanvas; SPoints: TFPointList; IsSolid: Boolean);
var
  I, N, PrevX, PrevY: Integer;
  P: TPoint;
  PDXF: TFPoint;
begin
  if SPoints.Count = 0 then
    Exit// Error
  else
  begin
    N := 2;
    if IsSolid then
      N := 1;
    I := 0;
    while I < SPoints.Count -1 do
    begin
      PDXF := SPoints[I];
      P := GetPoint(PDXF);
      PrevX := P.X;
      PrevY := P.Y;
      ACanvas.MoveTo(P.X, P.Y);
      PDXF := SPoints[I+1];
      P := GetPoint(PDXF);
      // For correct drawing and scaling dots
      if (P.X = PrevX) and (P.Y = PrevY) then
        ACanvas.LineTo(P.X, P.Y + 1)
      else
        ACanvas.LineTo(P.X, P.Y);
      Inc(I, N);
    end;
  end;
end;

procedure CopyEDataToList(const APoints: TFPointList; const ADataPointer: Pointer;
  const ADataCount: Integer);
var
  vPoint: PFPoint;
  vPointTo: TFPoint;
  I: Integer;
begin
  vPoint := ADataPointer;
  for I := 0 to ADataCount - 1 do
  begin
    vPointTo := vPoint^;
    APoints.Add(vPointTo);
    Inc(vPoint);
  end;
end;

{ DrawLine

  Draws a line.
  EData.Point - starting point
  EData.Point1 - ending point. }
procedure DrawLine(ACanvas: TCanvas);
var
  I: Integer;
  IntPoints: TList;
  SingPoints, DottedSingPoints: TFPointList;
begin
  SingPoints := TFPointList.Create;
  DottedSingPoints := TFPointList.Create;

  SingPoints.Add(EData.Point);
  SingPoints.Add(EData.Point1);

  CopyEDataToList(DottedSingPoints, EData.DashDots, EData.DashDotsCount);

  if fmCADDLLdemo.cbUseWinLine.Checked then // Global coordinates
  begin
    if Lines.IsSolid then
      DrawGlobal(ACanvas, SingPoints, Lines.IsSolid)
    else
    begin
      Lines.Line(SingPoints[0], SingPoints[1], DottedSingPoints);
      DrawGlobal(ACanvas, DottedSingPoints, Lines.IsSolid);
    end;
  end
  else
  begin
    IntPoints := TList.Create;
    if DottedSingPoints.Count = 0 then
    begin  // Solid
      GetPointsList(SingPoints, IntPoints);
      DrawPolyLine(ACanvas.Handle, IntPoints);
    end
    else
    begin
      GetPointsList(DottedSingPoints, IntPoints);
      DrawPolyPolyLine(ACanvas.Handle, IntPoints);
    end;

    for I := 0 to IntPoints.Count - 1 do
      Dispose(IntPoints[I]);
    IntPoints.Free;
  end;

  // Clearing memory
  DottedSingPoints.Free;
  SingPoints.Free;
end;

{ DrawSolid

  4 points in EData
  The order in FCADHandle file is 0-1-3-2. }
procedure DrawSolid(ACanvas: TCanvas);
var
  FSolid: array[0..3] of TPoint;
begin
  ACanvas.Pen.Width := 0;
  ACanvas.Brush.Style := bsSolid;
  ACanvas.Brush.Color := ACanvas.Pen.Color;
  FSolid[0] := GetPoint(EData.Point);
  FSolid[1] := GetPoint(EData.Point1);
  FSolid[2] := GetPoint(EData.Point2);
  FSolid[3] := GetPoint(EData.Point3);
  Polygon(ACanvas.Handle, FSolid[0], 4);
  ACanvas.Brush.Style := bsClear;
end;

procedure DrawArrow(ACanvas: TCanvas; ASegmentPoints: TFPointList);
var
  I: Integer;
  FSolid: array[0..3] of TPoint;
begin
  if (ASegmentPoints.Count < 0) or (ASegmentPoints.Count mod 4 <> 0) then
    Exit;
  ACanvas.Pen.Width := 0;
  ACanvas.Brush.Style := bsSolid;
  ACanvas.Brush.Color := ACanvas.Pen.Color;
  I := 0;
  while I < ASegmentPoints.Count do
  begin
    FSolid[0] := GetPoint(ASegmentPoints[I]);
    FSolid[1] := GetPoint(ASegmentPoints[I+1]);
    FSolid[2] := GetPoint(ASegmentPoints[I+2]);
    FSolid[3] := GetPoint(ASegmentPoints[I+3]);
    Polygon(ACanvas.Handle, FSolid[0], 4);
    Inc(I, 4);
  end;
  ACanvas.Brush.Style := bsClear;
end;


procedure Draw3DFace(ACanvas: TCanvas);

  procedure Edge(const P1, P2: TFPoint);
  var
    Pt: array[0..1] of TPoint;
  begin
    Pt[0] := GetPoint(P1);
    Pt[1] := GetPoint(P2);
    Polygon(ACanvas.Handle, Pt[0], 2);
  end;

begin
  ACanvas.Pen.Width := 0;
  ACanvas.Brush.Style := bsSolid;
  ACanvas.Brush.Color := ACanvas.Pen.Color;
  if EData.Flags and 1 = 0 then
    Edge(EData.Point, EData.Point1);
  if EData.Flags and 2 = 0 then
    Edge(EData.Point1, EData.Point2);
  if EData.Flags and 4 = 0 then
    Edge(EData.Point2, EData.Point3);
  if EData.Flags and 8 = 0 then
    Edge(EData.Point3, EData.Point);
  ACanvas.Brush.Style := bsClear;
end;

{ DrawPoly

  Draws a (poly)line in global coordinates.
  EData.Count - number of polyline vertices
  EData.Points - pointer to point array.    }
procedure DrawPoly(ACanvas: TCanvas);
var
  I: Integer;
  P: PFPoint;
  IntPoints: TList;
  SingPoints, DottedSingPoints, SegmentPoints: TFPointList;

begin
  SingPoints := TFPointList.Create;
  DottedSingPoints := TFPointList.Create;
  SegmentPoints := TFPointList.Create;

  CopyEDataToList(SingPoints, EData.Points, EData.Count);

  P := EData.Points;
  Inc(P,EData.Count);

  CopyEDataToList(SegmentPoints, P, EData.CountPointOfSegments);

  CopyEDataToList(DottedSingPoints, EData.DashDots, EData.DashDotsCount);

  if fmCADDLLdemo.cbUseWinLine.Checked then // Global coordinates
  begin
    if Lines.IsSolid then
      DrawGlobal(ACanvas, SingPoints, Lines.IsSolid)
    else
    begin
      Lines.Poly(SingPoints, DottedSingPoints, False);
      DrawGlobal(ACanvas, DottedSingPoints, Lines.IsSolid);
    end;
  end
  else
  begin
    IntPoints := TList.Create;
    if (DottedSingPoints.Count = 0) or
       (Assigned(EData.CADExtendedData) and not EData.CADExtendedData.IsDotted) then
    begin
      GetPointsList(SingPoints, IntPoints);
      DrawPolyLine(ACanvas.Handle, IntPoints);
    end
    else
    begin
      GetPointsList(DottedSingPoints, IntPoints);
      DrawPolyPolyLine(ACanvas.Handle, IntPoints);
    end;
    for I := 0 to IntPoints.Count - 1 do
      Dispose(IntPoints[I]);
    IntPoints.Free;
  end;

  DrawArrow(ACanvas, SegmentPoints);

  // Clearing memory
  DottedSingPoints.Free;
  SingPoints.Free;
  SegmentPoints.Free;
end;

{ GetSpline

  Gets a spline.  }
function GetSpline(ControlPoints, FitPoints: TFPointList; Knots: TsgSingleList; ChordLen: Single): TList;

  function Normalize(N, I: Integer; Param: Double): Double;
  var
    V1, D1, V2, D2: Double;
  begin
    if N = 0 then
    begin
      if (Single(Knots[I]) <= Param) and (Param < Single(Knots[I + 1])) then
        Result := 1
      else
        Result := 0;
    end
    else
    begin
      D1 := Single(Knots[I + N]) - Single(Knots[I]);
      V1 := (Param - Single(Knots[I])) * Normalize(N - 1, I, Param);
      if D1 = 0 then
        V1 := 0
      else
        V1 := V1 / D1;
      D2 := Single(Knots[I + N + 1]) - Single(Knots[I + 1]);
      V2 := (Single(Knots[I + N + 1]) - Param) * Normalize(N - 1, I + 1, Param);
      if D2 = 0 then
        V2 := 0
      else
        V2 := V2 / D2;
      Result := V1 + V2;
    end;
  end;

  function GetNURBS(Index: Integer; Param: Double): TFPoint;
  var
    I: Integer;
    Ni: Double;
  begin
    Result.X := 0;
    Result.Y := 0;
    Result.Z := 0;
    for I := Index - 3 to Index do
    begin
      Ni := Normalize(3, I, Param);
      Result.X := Result.X + ControlPoints[I].X * Ni;
      Result.Y := Result.Y + ControlPoints[I].Y * Ni;
      Result.Z := Result.Z + ControlPoints[I].Z * Ni;
    end;
  end;

  procedure AddPt(const Pt: TFPoint);
  var
    C: Integer;
    P: PInteger;
  begin
    C := Result.Count;
    Result.Count := Result.Count + cnstFPointSize;
    P := PInteger(Result.List);
    Inc(P,C);
    Move(Pt, P^, SizeOf(TFPoint));
  end;

var
  I: Integer;
  P, Prev: TFPoint;
  T: Double;
begin
  Result := TList.Create;
  P := ControlPoints[0];
  AddPt(P);
  Prev := P;
  for I := 3 to ControlPoints.Count - 1 do
  begin
    T := Knots[I];
    while  T < Knots[I + 1] do
    begin
      P := GetNURBS(I, T);
      T := T + 0.1; // the smaller value (for instance 0.01) the higher quality
      if (ChordLen > 0) and (abs(P.X - Prev.X) < ChordLen) and (abs(P.Y - Prev.Y) < ChordLen)
      then Continue;
      AddPt(P);
      Prev := P;
    end;
  end;
  P := ControlPoints[ControlPoints.Count - 1];
  AddPt(P);
end;

{ DrawSpline

  Draws a spline.  }
procedure DrawSpline(ACanvas: TCanvas);
var
  I: Integer;
  S: Single;
  vList: TList;
  Knots: TsgSingleList;
  P: PFPoint;
  vP: TFPoint;
  IntPoints: TList;
  SingPoints, DottedSingPoints, ControlPoints, FitPoints: TFPointList;

  PSinglePt: TFPoint;
  vDegree: Byte;

  procedure DrawCurrentSpline;
  var I: Integer;
  begin
    if fmCADDLLdemo.cbUseWinLine.Checked then // Global coordinates
    begin
      if Lines.IsSolid then
        DrawGlobal(ACanvas, SingPoints, Lines.IsSolid)
      else
      begin
        Lines.Curve(SingPoints, DottedSingPoints, False);
        DrawGlobal(ACanvas, DottedSingPoints, False);
      end;
    end
    else                                   // Display coordinates
    begin
      IntPoints := TList.Create;
      if Lines.IsSolid then
      begin
        GetPointsList(SingPoints, IntPoints);
        DrawPolyLine(ACanvas.Handle, IntPoints);
      end
      else
      begin
        Lines.Curve(SingPoints, DottedSingPoints, False);
        GetPointsList(DottedSingPoints, IntPoints);
        DrawPolyPolyLine(ACanvas.Handle, IntPoints);
      end;
      for I := 0 to IntPoints.Count - 1 do Dispose(IntPoints[I]);
      IntPoints.Free;
    end;
  end;

begin
  vDegree := EData.Flags;
  SingPoints := TFPointList.Create;
  DottedSingPoints := TFPointList.Create;
  ControlPoints := TFPointList.Create;
  FitPoints := TFPointList.Create;
  Knots := TsgSingleList.Create;
  try         
    P := EData.Points;
    for I := 0 to EData.Count - 1 do
    begin
      vP := PFPoint(P)^;
      ControlPoints.Add(vP);
      Inc(P);
    end;
    for I := 0 to PInteger(P)^ - 1 do
    begin
      Inc(P);
      vP := PFPoint(P)^;
      FitPoints.Add(vP);
    end;
    Inc(P);
    for I := 0 to PInteger(P)^ - 1 do
    begin
      Inc(P);
      S := P^.X;
      Knots.Add(S);
    end;

    if bSplittedSplines or (FitPoints.Count > ControlPoints.Count) or
       (vDegree <> 3) then
    begin
      CopyEDataToList(SingPoints, EData.DashDots, EData.DashDotsCount);
      IntPoints := TList.Create;
      try
        GetPointsList(SingPoints, IntPoints);
        if Lines.IsSolid then
          DrawPolyLine(ACanvas.Handle, IntPoints)
        else
          DrawPolyPolyLine(ACanvas.Handle, IntPoints);
      finally
        for I := 0 to IntPoints.Count - 1 do
          Dispose(IntPoints[I]);
        IntPoints.Free;
      end;
    end
    else
    begin
      vList := GetSpline(ControlPoints, FitPoints, Knots, 0);
      try
        EData.Points := PFPoint(vList.List);
        EData.Count := vList.Count div cnstFPointSize;
        P := EData.Points;
        for I := 0 to EData.Count-1 do
        begin
          PSinglePt := PFPoint(P)^;
          SingPoints.Add(PSinglePt);
          Inc(P);
        end;
        DrawCurrentSpline;
      finally
        vList.Free;
      end;
    end;
    
  finally
    // Clearing memory
    DottedSingPoints.Free;
    ControlPoints.Free;
    SingPoints.Free;
    FitPoints.Free;
    Knots.Free;
  end;
end;

{ DrawText

  Draws a text. }
procedure DrawText(ACanvas: TCanvas);
var
  P: TPoint;
  vFontName: string;
  S: WideString;
  PPts: PFPoint;
  I, J, CntI, Cnt: Integer;
  IntPoints, Counts: TsgIntegerList;
  vPts: PInteger;
  vDC: HDC;

  procedure DoFontFromData;
  var
    I, Len: Integer;
    vLogFont: TLogFont;
  begin
    GetObject(ACanvas.Font.Handle, SizeOf(vLogFont), @vLogFont);
    FillChar(vLogFont.lfFaceName, LF_FACESIZE, 0);
    vFontName := string(EData.FontName);
    Len := Length(vFontName);
    if Len = 0 then
      vLogFont.lfFaceName := 'Arial'
    else// setting real TrueType font
    begin
      for I := 0 to LF_FACESIZE - 1 do
        if (I < Len) and (vFontName[I + 1] <> '.') then
          vLogFont.lfFaceName[I] := vFontName[I + 1]
        else
          Break;
    end;
    vLogFont.lfHeight := Round(fWinTextHeightFactor * EData.FHeight * fmCADDLLdemo.Scale);
    if vLogFont.lfHeight = 0 then vLogFont.lfHeight := 1;
    vLogFont.lfWidth := Abs(Round(0.64 * EData.FHeight * EData.FScale * fmCADDLLdemo.Scale));
    if vLogFont.lfWidth = 0 then vLogFont.lfWidth := 1;
    vLogFont.lfEscapement := Round(EData.Rotation * 10);	// in tenths of degrees
    while vLogFont.lfEscapement < 0 do
      Inc(vLogFont.lfEscapement, 3600);
    vLogFont.lfOrientation := vLogFont.lfEscapement;
    ACanvas.Font.Handle := CreateFontIndirect(vLogFont);
  end;

begin
  if bTextByCurves and (EData.DashDotsCount > 0) then
  begin
    IntPoints := TsgIntegerList.Create;
    Counts := TsgIntegerList.Create;
    try
      PPts := EData.DashDots;
      I := 0;
      while I < EData.DashDotsCount do
      begin
        Cnt := PInteger(PPts)^;
        Counts.Add(Cnt);
        Inc(PPts);
        for J := 1 to Cnt do
        begin
          P := GetPoint(PPts^);
          IntPoints.Add(P.X);
          IntPoints.Add(P.Y);
          Inc(PPts);
        end;
        Inc(I);
      end;
      if EData.Flags and 1 <> 0 then
      begin
        // SHX text
        vDC := ACanvas.Handle;
        vPts := PInteger(IntPoints.List);
        CntI := Counts.Count - 1;
        for I := 0 to CntI do
        begin
          Cnt := Integer(Counts.List[I]);
          if (Cnt = 2) and (vPts^ = PInteger(Integer(vPts) + 8)^) and
            (PInteger(Integer(vPts) + 4)^ = PInteger(Integer(vPts) + 12)^) then
            SetPixelV(vDC, vPts^, PInteger(Integer(vPts) + 4)^, ACanvas.Pen.Color)
          else
            Windows.Polyline(vDC, vPts^, Cnt);
          Inc(vPts, Cnt shl 1);
        end;
      end
      else
      begin
        // TTF text
        ACanvas.Brush.Style := bsSolid;
        ACanvas.Brush.Color := ACanvas.Pen.Color;
        Windows.PolyPolygon(ACanvas.Handle, IntPoints.List^, Counts.List^, Counts.Count);
        ACanvas.Brush.Style := bsClear;
      end;
    finally
      IntPoints.Free;
      Counts.Free;
    end;
  end
  else
  begin
    if EData.HAlign in [1,2,4] then
      P := GetPoint(EData.Point1)
    else
      P := GetPoint(EData.Point);
    DoFontFromData;
    ACanvas.Font.Color := ACanvas.Pen.Color;
    SetTextAlign(ACanvas.Handle, TA_BASELINE);
    ACanvas.Brush.Style := bsClear;
    SetString(S, EData^.Text, StrLen(EData^.Text));
    ACanvas.TextOut(P.X, P.Y, string(S));
  end;
end;

{ DrawArcAsPoly

  Draws a arc as poplyline. }
procedure DrawArcAsPoly(ACanvas: TCanvas; const P0,P1,P2,P3: TFPoint);
const
  NSegs = 16; // Number of segments for full ellipse
var
  CX,CY,A,B,AStart,AEnd,Delta: Double;
  S,C: Extended;        // Sin and Cos of current angle
  I,N: Integer;
  P: TFPoint;
  IntPoints: TList;
  SingPoints, DottedSingPoints :TFPointList;
  PSinglePt: TFPoint;

  function Angle(const Pt: TFPoint): Extended;
  begin
    Result := ArcTan2(Pt.Y - CY, Pt.X - CX);
    if Result < 0 then Result := Result + Pi;
    if Pt.Y < CY then Result := Result + Pi;
  end;

begin
  SingPoints := TFPointList.Create;
  DottedSingPoints := TFPointList.Create;

  CX := (P0.X + P1.X) / 2;      // Center X
  CY := (P0.Y + P1.Y) / 2;      // Center Y
  A := (P1.X - P0.X) / 2;       // Horizontal radius
  B := (P0.Y - P1.Y) / 2;       // Vertical radius
  AStart := Angle(P2);          // Start angle
  AEnd := Angle(P3);            // End angle

  if AEnd <= AStart then AEnd := AEnd + 2*Pi;
  N := Round((AEnd-AStart)/Pi*NSegs);   // Real number of segments depends on arc angle
  if N < 4 then N := 4;
  Delta := (AEnd-AStart)/(N-1);

  for I:=0 to N-1 do
  begin
    SinCos(AStart,S,C);
    P.X := CX + A*C;                    // Current segment X
    P.Y := CY + B*S;                    // Current segment Y

    PSinglePt.X := P.X;
    PSinglePt.Y := P.Y;
    PSinglePt.Z := 0.0;// for future version
    SingPoints.Add(PSinglePt);
    AStart := AStart + Delta;           // Next segment
  end;

  if fmCADDLLdemo.cbUseWinLine.Checked then // Global coordinates
  begin
    if Lines.IsSolid then
      DrawGlobal(ACanvas, SingPoints, Lines.IsSolid)
    else
    begin
      Lines.Curve(SingPoints, DottedSingPoints, False);
      DrawGlobal(ACanvas, DottedSingPoints, Lines.IsSolid);
    end;
  end
  else
  begin
    IntPoints := TList.Create;
    if Lines.IsSolid then
    begin
      GetPointsList(SingPoints, IntPoints);
      DrawPolyLine(ACanvas.Handle, IntPoints);
    end
    else
    begin
      Lines.Curve(SingPoints, DottedSingPoints, False);
      GetPointsList(DottedSingPoints, IntPoints);
      DrawPolyPolyLine(ACanvas.Handle, IntPoints);
    end;
    for I := 0 to IntPoints.Count - 1 do
      Dispose(IntPoints[I]);
    IntPoints.Free;
  end;

  DottedSingPoints.Free;
  SingPoints.Free;
end;

{ DrawArc

  Draws a arc.
  EData.Point, EData.Point1 - bounding rectangle
  EData.Point2 - starting point
  EData.Point3 - ending point
  For full ellipse, Point2 = Point3.             }
procedure DrawArc(ACanvas: TCanvas);
var
  P, P1, P2, P3: TPoint;
  vRect: TRect;
  vRatio, vMajorLength: Single;
begin
  if fmCADDLLdemo.cbSplitArcs.Checked then
  begin
    DrawArcAsPoly(ACanvas, EData.Point, EData.Point1, EData.Point2, EData.Point3);
    Exit;
  end;
  P := GetPoint(EData.Point);
  P1 := GetPoint(EData.Point1);
  P2 := GetPoint(EData.Point2);
  P3 := GetPoint(EData.Point3);
  //Windows.SetArcDirection(PCanvas.Handle, AD_COUNTERCLOCKWISE);
  if ((EData^.Point2.X <> EData^.Point3.X)
    or (EData^.Point2.Y <> EData^.Point3.Y)) and (P2.X = P3.X)
    and (P2.Y = P3.Y) then
  begin
    SetPixelV(ACanvas.Handle, P2.X, P2.Y, ACanvas.Pen.Color);
    Exit;
  end;
  if (fmCADDLLdemo.cbProhibitArcsAsCurves.Checked) and (EData.EntityType=1) then
  begin
    vRatio := EData.Ratio;
    P := GetPoint(EData.Point);
    P1 := GetPoint(MakeFPoint(EData.Point.X + EData.Point1.X, EData.Point.Y + EData.Point1.Y, 0));
    vMajorLength := Sqrt(Sqr(P1.X - P.X) + Sqr(P1.Y - P.Y));
    vRect.Left := P.X - Round(vMajorLength);
    vRect.Top := P.Y + Round(vMajorLength * vRatio);
    vRect.Right := P.X + Round(vMajorLength);
    vRect.Bottom := P.Y - Round(vMajorLength * vRatio);
    ACanvas.Arc(vRect.Left, vRect.Top, vRect.Right, vRect.Bottom, P2.X, P2.Y, P3.X, P3.Y);
  end
  else
    if (EData.StartAngle = EData.EndAngle) then
      SetPixelV(ACanvas.Handle, P2.X, P2.Y, ACanvas.Pen.Color)
    else
      ACanvas.Arc(P.X, P1.Y, P1.X, P.Y, P2.X, P2.Y, P3.X, P3.Y);
end;

{ DrawHatch

  Draws a hatch.
  EData.Count - number of polyline vertices
  EData.Points - pointer to point array }
procedure DrawHatch(ACanvas: TCanvas);
var
  I, SaveIndex, Index: Integer;
  P: PFPoint;
  PSinglePt: TFPoint;
  vPoly: TsgIntegerList;
  SingPoints: TFPointList;
  vRgn, vMainRGN: HRGN;
  P1: TPoint;
begin
  if EData.Flags and 16 <> 0 then             // hatch is SOLID
  begin
    vPoly := TsgIntegerList.Create;
    vMainRGN := CreateRectRgn(0, 0, 0, 0);// FCanvas.ClipRect.Right, FCanvas.ClipRect.Bottom);
    SaveIndex := SaveDC(ACanvas.Handle);
    try
      P := EData.Points;
      Index := 0;// this is a counter of points for each boundary
      I := 0;    // this is a counter of boundaries themselves
      while I <= EData.Count do
      begin
        if Index = 0 then
        begin
          if I <> 0 then
          begin
            vRGN := CreatePolygonRgn(vPoly.List^, vPoly.Count shr 1, ALTERNATE);
            CombineRgn(vMainRGN, vRGN, vMainRGN,  RGN_XOR);
            DeleteObject(vRGN);
            vPoly.Clear(False);
            if I = EData.Count then
              Break;
          end;
          Index := PInteger(P)^;
          Inc(I);
        end
        else
        begin
          Dec(Index);
          P1 := GetPoint(P^);
          vPoly.Add(P1.X);
          vPoly.Add(P1.Y);
        end;
        Inc(P);
      end;
      GetRegionData(vMainRgn,0,nil);
      ACanvas.Brush.Style := bsSolid;
      ACanvas.Brush.Color := EData.Color;
      FillRgn(ACanvas.Handle, vMainRGN, ACanvas.Brush.Handle);
    finally
      RestoreDC(ACanvas.Handle, SaveIndex);
      DeleteObject(vMainRGN);
      vPoly.Free;
    end;
    Exit;
  end;
  SingPoints := TFPointList.Create;
  P := EData.DashDots;
  for I := 0 to EData.DashDotsCount - 1 do
  begin
    PSinglePt.X := P^.X;
    PSinglePt.Y := P^.Y;
    PSinglePt.Z := P^.Z;
    SingPoints.Add(PSinglePt);
    Inc(P);
  end;
  DrawGlobal(ACanvas, SingPoints, False);
  // Clearing memory
  SingPoints.Free;
end;

{ DrawImageEnt

  Draw a bmp image}
procedure DrawImageEnt(ACanvas: TCanvas);
var
  P1,P2: TPoint;
//  P,P3: TPoint;
  bmp: TBitmap;
  vStr: TMemoryStream;
  I: Integer;
  vRect: TRect;
begin
//  P := GetPoint(EData.Point);
  P1 := GetPoint(EData.Point1);
  P2 := GetPoint(EData.Point2);
//  P3 := GetPoint(EData.Point3);
  vRect.TopLeft := P2;
  vRect.BottomRight := P1;
  if vRect.Right < vRect.Left then
  begin
    I := vRect.Right;
    vRect.Right := vRect.Left;
    vRect.Left := I;
  end;
  if vRect.Bottom < vRect.Top then
  begin
    I := vRect.Bottom;
    vRect.Bottom := vRect.Top;
    vRect.Top := I;
  end;
  vStr := TMemoryStream.Create;
  bmp := TBitmap.Create;
  try
    vStr.Write(EData.Ticks^, EData.Handle);
    vStr.Position := 0;
    bmp.LoadFromStream(vStr);
    ACanvas.StretchDraw(vRect, bmp);
  finally
    vStr.Free;
    bmp.Free;
  end;
end;

{ BeginViewport

  Creates a clipping region according to the VIEWPORT's boundary.
  Makes necessary actions before drawing the VIEWPORT and his "contents".    }
procedure BeginViewport(ACanvas: TCanvas);
var
  L: TsgIntegerList;
  P: PFPoint;
  TP: TPoint;
  I, J, K: Integer;
  MainRgn, Rgn: HRgn;
  R: TRect;
begin
  R.TopLeft := GetPoint(EData.Point);
  R.BottomRight := GetPoint(EData.Point1);
  if R.Left > R.Right then SwapInts(R.Left, R.Right);
  if R.Top > R.Bottom then SwapInts(R.Top, R.Bottom);
  SaveDC(ACanvas.Handle);
  if EData.Count = 0 then
  begin
    MainRgn := CreateRectRgnIndirect(R);
    if EData.Flags and 1 <> 0 then
      ACanvas.Rectangle(R.Left, R.Top, R.Right, R.Bottom);
  end
  else
  begin
    P := EData.Points;
    I := EData.Count;
    MainRgn := CreateRectRgn(0,0,0,0);
    L := TsgIntegerList.Create;
    try
      while I > 0 do
      begin
        L.Count := 0;
        K := PInteger(P)^;
        Inc(P);
        for J := 0 to K - 1 do
        begin
          TP := GetPoint(P^);
          Inc(P);
          L.Add(TP.X);
          L.Add(TP.Y);
        end;
        Rgn := CreatePolygonRgn(L.List^, K, ALTERNATE);
        CombineRgn(MainRgn, MainRgn, Rgn, RGN_XOR);
        DeleteObject(Rgn);
        Dec(I);
      end;
    finally
      L.Free;
    end;
  end;
  TP := Point(0,0);
  LPtoDP(ACanvas.Handle, TP, 1);
  OffsetRgn(MainRgn, TP.X, TP.Y);
  SelectClipRgn(ACanvas.Handle, MainRgn);
  DeleteObject(MainRgn);
end;

{ EndViewport

  Makes necessary actions after drawing the VIEWPORT and his "contents".    }
procedure EndViewport(ACanvas: TCanvas);
begin
  RestoreDC(ACanvas.Handle, -1);
end;

{ DoLines

  Creates the list of ticks.
  For complex types of lines:
     EData.TicksCount =0 and EData.DashDotsCount <> 0
   }
procedure DoLines;
type
  PSingle = ^Single;
var
  I: Integer;
  P: PSingle;
  vPElement: PsgLTypeElement;
begin
  Lines.Clear;
  Lines.Scale := EData.Rotation*LScale;
  P := PSingle(EData.Ticks);
  for I := 0 to EData.TickCount - 1 do
  begin
    New(vPElement);
    FillChar(vPElement^, SizeOf(vPElement^), 0);
    vPElement^.Dash := P^;
    Lines.Add(vPElement);
    Inc(P);
  end;
  Lines.Loaded(nil);
end;

{ DoDraw

  Called from DXFEnum for each entity in FCADHandle file
  Data points to entity data
  Param is user-defined parameter. }
procedure DoDraw(Data: PcadData; var Param); stdcall;
var
  vCanvas: TCanvas;
begin
  vCanvas := TfmCADDLLdemo(Param).Canvas;
  // if DXFVisible(FCADHandle, Data.Layer) = 0 then
  //   Exit; // invisible layers are ignored
  // MessageBox(0, PChar(Format('%d', [SizeOf(Data)])), 'a', 0);
  if not fmCADDLLdemo.cbDimensions.Checked and (Data.Dimension <> 0) then Exit;
  EData := Data;
  vCanvas.Pen.Color := Data.Color;      // entity color
  vCanvas.Pen.Width := Round(Data.Thickness * fmCADDLLdemo.Scale);
  DoLines;
  case EData.Tag of
    DXF_LINE:                           DrawLine(vCanvas);
    DXF_SOLID:          	              DrawSolid(vCanvas);
    DXF_3DFACE:                         Draw3DFace(vCanvas);
    DXF_CIRCLE,DXF_ARC,DXF_ELLIPSE:     DrawArc(vCanvas);
    DXF_POLYLINE..DXF_LWPOLYLINE:       DrawPoly(vCanvas);
    DXF_SPLINE:                         DrawSpline(vCanvas);
    DXF_TEXT, DXF_ATTDEF, DXF_ATTRIB:   DrawText(vCanvas);
    DXF_POINT:		                      DrawPoint(vCanvas);
    DXF_HATCH:                          DrawHatch(vCanvas);
    DXF_IMAGE_ENT:                      DrawImageEnt(vCanvas);
    DXF_BEGIN_INSERT:                   IsInsideInsert := True;
    DXF_END_INSERT:                     IsInsideInsert := False;
    DXF_BEGIN_VIEWPORT:                 BeginViewport(vCanvas);
    DXF_END_VIEWPORT:                   EndViewport(vCanvas);
  end;
end;

procedure ScaleRectangle(const AScale: Double; const APos: TF2DPoint; var ARect: TF2DRect);
var
  W, H: Double;
begin
  W := (ARect.Right - ARect.Left) * AScale;
  H := (ARect.Bottom - ARect.Top) * AScale;
  ARect.Left := (ARect.Left * AScale) + (-APos.X * AScale) + APos.X;
  ARect.Top := (ARect.Top * AScale) + (-APos.Y * AScale) + APos.Y;
  ARect.Right := ARect.Left + W;
  ARect.Bottom := ARect.Top + H;
end;

procedure FitToWindow(const ClientR: TRect; AHWRatio: Double;
  var ARect: TF2DRect);
var
  vD, vdX, vdY: Double;
begin
	ARect.Left := ClientR.Left;
	ARect.Top := ClientR.Top;
	ARect.Right := ClientR.Right;
	ARect.Bottom := ClientR.Bottom;
  vD := (ARect.Right - ARect.Left) * AHWRatio;
  if (vD > (ARect.Bottom - ARect.Top)) then
    ARect.Right := ARect.Left + (ARect.Bottom - ARect.Top) / AHWRatio
  else
    ARect.Bottom := ARect.Top + vD;
	vdX := ((ClientR.Right - ClientR.Left) - (ARect.Right - ARect.Left)) / 2;
	vdY := ((ClientR.Bottom - ClientR.Top) - (ARect.Bottom - ARect.Top)) / 2;
  ARect.Left := ARect.Left + vdX;
  ARect.Right := ARect.Right + vdX;
  ARect.Top := ARect.Top + vdY;
  ARect.Bottom := ARect.Bottom + vdY;
end;

{ TfmCADDLLdemo.pbDrawingPaint

  Use bit-coded DXFEnum flag =4 for setting a way of cutting a viewport's
  contents to fit a "rectangular" VIEWPORT, without using Windows API REGIONS.
  If you have no possibility to use Windows API REGIONS
  (for example, you use OpenGL or something similar), set this flag.  }
procedure TfmCADDLLdemo.FormPaint(Sender: TObject);
begin
  if FCADHandle <> 0 then
  begin
    Lines.Scale := LScale;
    Canvas.Brush.Color := Color;
    Canvas.FillRect(Canvas.ClipRect);
    CADEnum(FCADHandle, Ord(sbAllLayers.Checked) or (Integer(bTextByCurves) shl 3)
      or (Integer(b3DFaceForAcis) shl 4), @DoDraw, Self);        // use DXFEnum for import as a linear "metafile"
  end;
end;

function TfmCADDLLdemo.GetAbsHeight: Double;
begin
  Result := Abs(FExtentsCAD.Top - FExtentsCAD.Bottom);
end;

function TfmCADDLLdemo.GetAbsWidth: Double;
begin
  Result := Abs(FExtentsCAD.Right - FExtentsCAD.Left);
end;

function TfmCADDLLdemo.GetDrawRect: TF2DRect;
begin
  Result := FDrawRect;
end;

function TfmCADDLLdemo.GetScale: Double;
begin
  Result := (FDrawRect.Right - FDrawRect.Left) / AbsWidth;
end;

function TfmCADDLLdemo.GetVewport: TRect;
begin
  Result := ClientRect;
  Result.Top := pnlOptions.BoundsRect.Bottom;
end;

procedure TfmCADDLLdemo.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if FCADHandle <> 0 then
    CADClose(FCADHandle);
end;

procedure TfmCADDLLdemo.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button in [mbRight, mbMiddle] then
  begin
    FDown := True;
    FDownPos := Point(X, Y);
    Cursor := crHandPoint;
    Perform(WM_SETCURSOR, Handle, HTCLIENT);
  end;
  if [ssDouble, ssMiddle] * Shift = [ssDouble, ssMiddle] then
    DoFit;
end;

procedure TfmCADDLLdemo.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  dx, dy: Integer;
begin
  if FDown and ([ssRight, ssMiddle] * Shift <> []) then
  begin
    dx := X - FDownPos.X;
    dy := Y - FDownPos.Y;
    FDrawRect.Left := FDrawRect.Left + dx;
    FDrawRect.Right := FDrawRect.Right + dx;
    FDrawRect.Top := FDrawRect.Top + dy;
    FDrawRect.Bottom := FDrawRect.Bottom + dy;
    FDownPos := Point(X, Y);
    Invalidate;
  end;
end;

procedure TfmCADDLLdemo.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if FDown and (Button in [mbRight, mbMiddle]) then
  begin
    FDown := False;
    Cursor := crDefault;
  end;
end;

procedure TfmCADDLLdemo.FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  Handled := DoScale(4/5, MousePos);
  if Handled then
    Invalidate;
end;

procedure TfmCADDLLdemo.FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  Handled := DoScale(5/4, MousePos);
  if Handled then
    Invalidate;
end;

procedure TfmCADDLLdemo.cbSplitSplinesClick(Sender: TObject);
begin
  bSplittedSplines := cbSplitSplines.Checked;
  Invalidate
end;

procedure TfmCADDLLdemo.cbLayersDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var C: TColor;
begin
  C := TColor(cbLayers.Items.Objects[Index]);
  with cbLayers.Canvas do
  begin
    Brush.Color := clWindow;
    FillRect(Rect);
    if C < 0 then Font.Color := clGray else Font.Color := clBlack;
    TextRect(Rect, Rect.Left+16, Rect.Top, cbLayers.Items[Index]);
    Brush.Color := C and MaxInt;
    Rectangle(Rect.Left+2, Rect.Top+2, Rect.Left+12, Rect.Top+12);
  end;
end;

procedure TfmCADDLLdemo.FormCreate(Sender: TObject);
begin
  Lines := TsgLines.Create;
  btnFitToWindow.Enabled := False;
  cbSplitSplines.Checked := True;
  cbTextAsCurves.Checked := True;
  UpdateExtents;
  InitSHX(Pointer(@CADSetSHXOptions));
end;

procedure TfmCADDLLdemo.FormDestroy(Sender: TObject);
begin
  Lines.Free;
end;

procedure TfmCADDLLdemo.mmiDoubleBufferedClick(Sender: TObject);
begin
  DoubleBuffered := mmiDoubleBuffered.Checked;
  Invalidate;
end;

procedure TfmCADDLLdemo.mmiFontSettingsClick(Sender: TObject);
begin
  FontSettingsDlg.Visible := True;
end;

procedure TfmCADDLLdemo.mmiOpenClick(Sender: TObject);
const S: array[0..1] of string = ('Inches','Millimeters');
var
  I, Cnt: Integer;
  C: Cardinal;
  Data: TcadData;
  vLayoutName: string;
  vWStr: WideString;
begin
  try
    if mmiOpen.Checked then
      Exit;
    OpenDialog1.InitialDir := FCurrentDir;
    if not OpenDialog1.Execute then
      Exit;
    mmiOpen.Checked := True;
    if Length(OpenDialog1.FileName) > 1 then
      FCurrentDir := ExtractFilePath(OpenDialog1.FileName);
    if FCADHandle <> 0 then CADClose(FCADHandle);
    FCADHandle := CADCreate(Application.Handle, PChar(OpenDialog1.FileName));    // Open FCADHandle file
    if FCADHandle = 0 then Error;
    if CADUnits(FCADHandle,I) = 0 then Error;    // Drawing units (inches or millimeters)
    if CADLTScale(FCADHandle,LScale) = 0 then Error;

    UpdateExtents;

    lblMeasure.Caption := S[I];

    cbLayers.Items.Clear;
    Cnt := CADLayerCount(FCADHandle);
    FillChar(Data, SizeOf(Data), 0);
    for I := 0 to Cnt - 1 do
    begin
      CADLayer(FCADHandle, I, @Data);
      {  Data.flags bit code
      1 - Visible
      2 - Frozen
      4 - Locked
      8 - IsPlotting
      }
      C := Data.Color;
      if Data.Flags and 3 <> 1 then
        C := C or $80000000;
      SetString(vWStr, Data.Text, StrLen(Data.Text));
      cbLayers.Items.AddObject(string(vWStr), TObject(C));
    end;
    cbLayouts.Items.Clear;
    Cnt := CADLayoutCount(FCADHandle);
    for I := 0 to Cnt - 1 do
    begin
      SetLength(vLayoutName, 100);
      CADLayoutName(FCADHandle, I, PChar(vLayoutName), 100);
      cbLayouts.Items.Add(vLayoutName);
    end;
    if CADLayoutCurrent(FCADHandle, DWORD(I), False) = 1 then
      cbLayouts.ItemIndex := I;
    DoFit;
    cbProhibitArcsAsCurvesClick(nil);
    btnFitToWindow.Enabled := True;
    mmiOpen.Checked := False;
    mmiSaveAs.Enabled := True;
    Caption := cnstCaption + ' - [' + ExtractFileName(OpenDialog1.FileName) + ']';
  except
    on E: Exception do
    begin
      Caption := cnstCaption;
      mmiOpen.Checked := False;
      if E is ECADDLLException then
        MessageBox(Handle, PChar(E.Message), 'CAD DLL Error', MB_ICONERROR);
    end;
  end;
end;

procedure TfmCADDLLdemo.FitToWindowClick(Sender: TObject);
begin
  DoFit;
end;

procedure TfmCADDLLdemo.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfmCADDLLdemo.cbSplitArcsClick(Sender: TObject);
begin
  Invalidate;
end;

procedure TfmCADDLLdemo.cbTextAsCurvesClick(Sender: TObject);
begin
  bTextByCurves := cbTextAsCurves.Checked;
  Invalidate;
end;

procedure TfmCADDLLdemo.cb3DFaceForACISClick(Sender: TObject);
begin
  b3DFaceForAcis := CheckBox1.Checked;
  Invalidate;
end;

procedure TfmCADDLLdemo.cbUseWinLineClick(Sender: TObject);
begin
  Invalidate;
end;

procedure TfmCADDLLdemo.DoFit;
begin
  if FCADHandle <> 0 then
  begin
    FitToWindow(GetVewport, AbsHeight / AbsWidth, FDrawRect);
    Invalidate;
  end;
end;

function TfmCADDLLdemo.DoScale(const AScale: Double; ScreenPos: TPoint): Boolean;
var
  vDroppedDown: Boolean;
begin
  Result := False;
  vDroppedDown := False;
  if ActiveControl is TComboBox then
    vDroppedDown := TComboBox(ActiveControl).DroppedDown;
  if not vDroppedDown then
  begin
    ScreenPos := ScreenToClient(ScreenPos);
    ScaleRectangle(AScale, MakeF2DPoint(ScreenPos.X, ScreenPos.Y), FDrawRect);
    Result := True;
  end;
end;

procedure TfmCADDLLdemo.cbProhibitArcsAsCurvesClick(Sender: TObject);
begin
  // 1 => permit conversion of arcs to polyline
  CADProhibitCurvesAsPoly(FCADHandle, Ord(not fmCADDLLdemo.cbProhibitArcsAsCurves.Checked));
  Invalidate;
end;

procedure TfmCADDLLdemo.pmiShowAboutClick(Sender: TObject);
begin
  ShowAboutDlg('About CAD DLL');
end;

procedure TfmCADDLLdemo.UpdateExtents;
begin
  if FCADHandle <> 0 then
  begin
    CADGetBox(FCADHandle, FExtentsCAD.Left, FExtentsCAD.Right,
      FExtentsCAD.Top, FExtentsCAD.Bottom);
  end
  else
    FExtentsCAD := MakeF2DRect(0, 1, 1, 0);
end;

procedure TfmCADDLLdemo.cbLayoutsChange(Sender: TObject);
var
  I: DWORD;
begin
  if FCADHandle <> 0 then
  begin
    I := cbLayouts.ItemIndex;
    CADLayoutCurrent(FCADHandle, I, True);
    UpdateExtents;
    DoFit;
  end;
end;

function GenerateXMLParams(const AFileName, AFileExt: string;
  const AAbsWidth, AAbsHeight: Single;
  const ADefaultColor, ABackground: TColor): string;
var
  vGraphicsParams, vCADParametrs, vDrawRect: IXMLNode;
  vXMLExportParams: IXMLDocument;
  vStream: TMemoryStream;
  vBytesSize: Integer;
{$IFDEF UNICODE}
  vBytes: TBytes;
{$ENDIF}
begin
  Result := '';
  vXMLExportParams := TXMLDocument.Create(nil);
  try
    vXMLExportParams.Active := false;
    vXMLExportParams.XML.Text := '<ExportParams version="1.0"/>';
    vXMLExportParams.Active := True;
    vXMLExportParams.DocumentElement.AddChild('Filename').Text := AFileName;
    vXMLExportParams.DocumentElement.AddChild('Ext').Text := AFileExt;

    vGraphicsParams := vXMLExportParams.DocumentElement.AddChild('GraphicParametrs');
    vGraphicsParams.AddChild('PixelFormat').Text := IntToStr(Integer(pf24bit));
    vGraphicsParams.AddChild('Width').Text := IntToStr(Round(AAbsWidth));
    vGraphicsParams.AddChild('Height').Text := IntToStr(Round(AAbsHeight));

    vGraphicsParams.AddChild('DrawMode').Text := '0';
    vDrawRect := vGraphicsParams.AddChild('DrawRect');
    vDrawRect.Attributes['Left'] := '0';
    vDrawRect.Attributes['Top'] := '0';
    vDrawRect.Attributes['Right'] := IntToStr(Round(AAbsWidth));
    vDrawRect.Attributes['Bottom'] := IntToStr(Round(AAbsHeight));

    vCADParametrs := vXMLExportParams.DocumentElement.AddChild('CADParametrs');
    vCADParametrs.AddChild('XScale').Text := '1';
    vCADParametrs.AddChild('BackgroundColor').Text := IntToStr(ABackground);
    vCADParametrs.AddChild('DefaultColor').Text := IntToStr(ADefaultColor);

    vStream := TMemoryStream.Create;
    try
      vXMLExportParams.Encoding := 'utf-8';
      vXMLExportParams.SaveToStream(vStream);
      vStream.Position := 0;
      vBytesSize := vStream.Size;
{$IFDEF UNICODE}
      try
        SetLength(vBytes, vBytesSize);
        vStream.Read(vBytes[0], vBytesSize);
        Result := TEncoding.UTF8.GetString(vBytes);
      finally
        SetLength(vBytes, 0);
      end;
{$ELSE}
      SetLength(Result, vBytesSize);
      vStream.Read(Result[1], vBytesSize);
{$ENDIF}
    finally
      vStream.Free;
    end;
  finally
    vXMLExportParams := nil;
  end;
end;


procedure TfmCADDLLdemo.mmiSaveAsClick(Sender: TObject);
const
  cnstBufSize = 1024;
var
  vFileName, vExt, vExportParams: string;
  vFileExts: TStringList;
  vKoef: Double;
  vMsg: array[1..cnstBufSize] of Char;

  function SetImageParam(APixelFormat: TPixelFormat = pf24bit; AWidth: Integer = 0;
    AHeigth: Integer = 0): PsgImageParam;
  begin
    New(Result);
    Result^.Heigth := AHeigth;
    Result^.Width := AWidth;
    Result^.PixelFormat := APixelFormat;
  end;

begin
  if (FCADHandle = 0) or not SaveDialog.Execute then Exit;
  vFileName := SaveDialog.FileName;
  vExt := ExtractFileExt(vFileName);
  if vExt = '' then
  begin
    vFileExts := TStringList.Create;
    try
      vFileExts.LineBreak := '|';
      vFileExts.Text := SaveDialog.Filter;
      vExt := vFileExts[SaveDialog.FilterIndex * 2 - 1];
      Delete(vExt, 1, 1);
      vFileName := vFileName + vExt;
    finally
      vFileExts.Free;
    end;
  end;

  if GetExportFormat(vExt) in [efBitmap..efPng] then
    vKoef := Min(800/AbsWidth, 600/AbsHeight)
  else
    vKoef := 1;

  vExportParams := WideString(GenerateXMLParams(vFileName, vExt,
    vKoef*AbsWidth, vKoef*AbsHeight,
    clBlack, clWhite));
  if Length(vExportParams) > 0 then
    if SaveCADtoFileWithXMLParams(FCADHandle, PChar(vExportParams), nil) <> 0 then
      Application.MessageBox(PChar('File saved as: ' + vFileName), '', MB_OK)
    else
    begin
      GetLastErrorCAD(@vMsg[1], cnstBufSize);
      Application.MessageBox(PChar(@vMsg[1]), 'CAD DLL Error', MB_OK);
    end;
end;


initialization
{$IFDEF MEMCHK}
  MemChk
{$ENDIF}

end.
